----------------------------------------------------------------------------------
-- Company: UQ School of ITEE
-- Engineer: Ian Clough
-- 
-- Create Date:    14:05:25 03/09/2010 
-- Design Name: 
-- Module Name:    codec.vhd - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.numeric_std.all;
---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity codec is
    Port ( SCLK : in  STD_LOGIC;
           CLK : in  STD_LOGIC;
           ADC_DONE : in  STD_LOGIC;
           ADC_SYNC : in  STD_LOGIC;
			  SELECTOR	: in STD_LOGIC;
			  DATA_IN1	:	in std_logic_vector(11 downto 0);
			  DATA_IN2	:	in std_logic_vector(11 downto 0);
			  DATA_OUT : out std_logic_vector(11 downto 0);
			  DAC_START : out  STD_LOGIC;
           ADC_START : out  STD_LOGIC);
end codec;

architecture Behavioral of codec is
signal startdac: std_logic := '0';
signal startadc: std_logic :='0';
signal dummy1 : std_logic_vector(11 downto 0);
signal dummy2 : std_logic_vector(11 downto 4);
signal DATA_IN : std_logic_vector(11 downto 0);
signal ctr1 : integer range 0 to 7:=5; -- to control pulse width
signal ctr2 : integer range 0 to 7:=5; -- to control pulse width
signal a1 : integer range 0 to 4095; --OK		--Sep 2 2011
--signal a1 : integer range 0 to 8191; --OK
signal a2 : integer range 0 to 16383;
TYPE StateType1 IS (ADC_Idle, ADC_HIGH, ADC_LOW);
signal adc_current_state, adc_next_state : StateType1;
TYPE StateType2 IS (DAC_Idle,DAC_HIGH,DAC_LOW);
signal dac_current_state,dac_next_state : StateType2;
begin
--
	selection:process(clk)
	begin
	if(clk'event and clk ='1') then
		if (SELECTOR = '0') then
		DATA_IN <= DATA_IN1;
		else 
		DATA_IN <= DATA_IN2;
		end if;
	end if;
	end process selection;
	
	adc_run:process(sclk)
	begin
	if (sclk' event and sclk = '1') then
		if(startadc = '0') then
		startadc <='1';
		else 
		startadc <='0';
		end if;
	end if;
end process adc_run;

	dac_run:process(sclk)
	begin
	if (sclk' event and sclk = '0') then
		if(startdac = '0') then
		startdac <='1';
		else 
		startdac <='0';
		end if;
	end if;
	end process dac_run;
	
read_adc:process(clk)
	begin
if (clk' event and clk ='1' ) then	-- 
		if (startadc  = '0') then
		adc_current_state <= ADC_Idle;
		else 
		adc_current_state <= adc_next_state;
		end if;	-- end if (mode ='1')
case adc_current_state is
	when ADC_Idle =>
	ADC_START <='0';
	ctr1 <=5; 
	if (startadc = '1')then
	adc_next_state <= ADC_HIGH;
	else
	adc_next_state <= ADC_Idle;
	end if;
--	startadc <='1';
	when ADC_HIGH =>
	ADC_START <='0';
	ctr1 <= ctr1 - 1;	-- control "convert" pulse width
		if(ctr1 = 0) then
		adc_next_state <= ADC_LOW;
		else 
		ADC_START <='1';	-- start conversion
		adc_next_state <= ADC_HIGH;
	end if;
	
	when ADC_LOW =>
	ctr1 <=0;
	ADC_START <='0';
	if (ADC_DONE = '0' and ADC_SYNC = '1') then
	a1 <= to_integer(unsigned(DATA_IN));
		if (a1*28/16)>4095 then
		a2 <=4095;
		else
		a2 <= a1*28/16;
		end if;
	
	end if;
	adc_next_state <= ADC_LOW;
		
	when OTHERS =>
	adc_next_state <= ADC_Idle;
	
end case;
end if;	--end if (rst='1')
end process read_adc;	
	
write_dac:process(clk)
	begin
if (clk' event and clk ='1' ) then	-- 
		if (startdac  = '0') then
		dac_current_state <= DAC_Idle;
		else 
		dac_current_state <= dac_next_state;
		end if;	-- end if (mode ='1')
case dac_current_state is
	when DAC_Idle =>
	DAC_START <='0';
	ctr2 <=5; 
	if (startdac = '1')then
--	DATA_OUT <= std_logic_vector(to_unsigned(a2,12));-- OK
	DATA_OUT <= std_logic_vector(to_unsigned(a1,12));-- OK 11August2011
	dac_next_state <= DAC_HIGH;
	else
	dac_next_state <= DAC_Idle;
	end if;
--	startdac <='1';
	when DAC_HIGH =>
	DAC_START <='1';	--sep 2 2011
	ctr2 <= ctr2 - 1;	-- control write pulse width
		if(ctr2 = 0) then
		DAC_START <='0';
		dac_next_state <= DAC_LOW;
		else 
		DAC_START <='1';	--convert data
		dac_next_state <= DAC_HIGH;
	end if;
	
	when DAC_LOW =>
	ctr2 <= 0;
	DAC_START <='0';
--	dac_next_state <= DAC_LOW;
	dac_next_state <= DAC_Idle;
		
	when OTHERS =>
	dac_next_state <= DAC_Idle;
	
end case;
end if;	--end if (rst='1')
end process write_dac;	
end Behavioral;

